import click
from snark.log import logger
from snark import config
from snark.client.pod_control import PodControlClient
from snark.cli.auth import login
from snark.token_manager import TokenManager


from subprocess import call
import os
import subprocess
from snark.cli.utils import get_proxy_command

def get_pod_key_path(username, pod_id):
    return os.path.join(config.POD_KEY_DIR_PATH, username, pod_id)

def install_pod_key(username, pod_id, private_key):
    key_file_path = get_pod_key_path(username, pod_id)
    if not os.path.exists(os.path.dirname(key_file_path)):
        os.makedirs(os.path.dirname(key_file_path))
    with open(key_file_path, 'w') as f:
        os.chmod(key_file_path, 0o600)
        f.write(private_key)

@click.command()
@click.argument('pod_id', default='my_pod')
@click.option('--port_forwarding', '-L', default='', help='Port forwarding')
@click.option('--jupyter', '-j', flag_value=True, default=False, help='Jupyter running')
@click.pass_context
def attach(ctx, pod_id, port_forwarding, jupyter):
   """ Connect to a started pod """
   if not TokenManager.is_authenticated():
       ctx.invoke(login)
   connect(pod_id, port_forwarding=port_forwarding, jupyter=jupyter)

def connect(pod_id, port_forwarding="", jupyter=False):
    """ Connect to a started pod """
    logger.info("Connecting to the pod...")
    proxy, address, username, private_key, port = PodControlClient().get_connect_info(pod_id)

    install_pod_key(username, pod_id, private_key)
    key_file_path = get_pod_key_path(username, pod_id)
    if not port_forwarding == "":
        port_forwarding = "-L "+port_forwarding

    ssh_proxy = get_proxy_command(proxy)
    ssh_jupyter = ""
    if jupyter:
        ssh_jupyter = '-L 8888:localhost:8888 jupyter notebook --ip=0.0.0.0 --port=8888'

    command = ('ssh -i "{}" -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null\
               {} {}@{} -p {} {} {}')\
               .format(key_file_path, ssh_proxy, username, address, port, port_forwarding, ssh_jupyter)

    os.system(command)



import json
import os
import snark
import time
from snark import config
from snark.log import logger
from snark.client.base import SnarkHttpClient
from snark.exceptions import SnarkException
import pprint
from subprocess import Popen, PIPE
import subprocess

class StoreControlClient(SnarkHttpClient):
    """
    Controlling Snark Pods through rest api
    """
    def __init__(self):
        super(StoreControlClient, self).__init__()
        self.details = self.get_config()

    def s3cmd(self, type='ls', source='', target='', recursive = False, stream_output = True):
        snark = 'snark://'
        s3 = self.details['bucket']+'/'
        if 'snark://' in source:
            source = source.replace(snark, s3)

        if 'snark://' in target:
            target = target.replace(snark, s3)

        rec = ['--recursive'] if recursive else []
        target = [target] if len(target)>0 else []

        cmd = ['aws', 's3', type] + [source] + target + rec

        details = self.get_config()
        env = os.environ.copy()
        env['AWS_ACCESS_KEY_ID'] = details['access_key']
        env['AWS_SECRET_ACCESS_KEY'] = details['secret_key']
        env['AWS_SESSION_TOKEN'] = details['access_token']

        self.run_command(cmd, env, s3, snark)

    def run_command(self, command, env, s3, snark):
        process = Popen(command, stdout=subprocess.PIPE, env=env)
        while True:
            output = process.stdout.readline()
            output = output.decode('utf-8')
            if output == '' and process.poll() is not None:
                break
            if output:
                print(output.replace(s3, snark)[:-1])
        rc = process.poll()
        return rc

    def get_credentials(self):
        r = self.request(
            'GET', config.GET_CREDENTIALS_SUFFIX,
            endpoint=config.SNARK_HYPER_ENDPOINT,
        ).json()

        details = {
            "host_base": r['endpoint'],
            "host_bucket": r['endpoint'],
            "bucket": r['bucket'],
            "bucket_location": r['bucket_location'],
            "use_https": r['use_https'],
            "access_key" : r['access_key'],
            "secret_key": r['secret_key'],
            "access_token": r['session_token'],
            "expiration": r['expiration'],
            "signature_v2": False
        }
        self.save_config(details)
        return details

    def get_config(self):
        if not os.path.isfile(config.STORE_CONFIG_PATH):
            self.get_credentials()

        with open(config.STORE_CONFIG_PATH, 'r') as file:
            details = file.readlines()
            details = json.loads(''.join(details))

        if float(details['expiration'])<time.time():
            details = self.get_credentials()

        return details

    def save_config(self, details):
        with open(config.STORE_CONFIG_PATH, 'w') as file:
            file.writelines(json.dumps(details))

        #s3cmd_config = [str(key)+' = '+str(details[key]) for key in details]
        #with open(config.S3CMD_CONFIG_PATH, 'w') as file:
        #    file.writelines('\n'.join(s3cmd_config))



import click
from snark.log import logger
from snark.client.hyper_control import HyperControlClient
from snark.client.store_control import StoreControlClient
import yaml
from os import walk
import pprint
from tabulate import tabulate
import json
import os

def get_all_files(path = "./"):
    f = []
    for (dirpath, dirnames, filenames) in walk(path):
        f.append((dirpath, dirnames, filenames))
        #break
    #print(f)
    return f

@click.command()
@click.argument('source', default='')
@click.argument('target', default='')
@click.pass_context
def sync(ctx, source, target):
    """ Synchronizes files with Snark Storage """
    StoreControlClient().s3cmd('sync', source, target)

@click.command()
@click.argument('source', default='snark://')
@click.pass_context
def ls(ctx, source):
    """ List files of a specific folder """
    StoreControlClient().s3cmd('ls', source)

@click.command()
@click.argument('source', default='')
@click.argument('target', default='')
@click.option('--recursive', '-r', flag_value=bool, default=False, help='recursive upload')
@click.pass_context
def cp(ctx, source, target, recursive):
    """ Copy files from/to Snark Storage """
    cmd = ''
    if 'snark://' in source:
        cmd = 'get'
    if 'snark://' in target:
        cmd = 'put'
    if 'snark://' in source and 'snark://' in target:
        cmd = 'cp'
    cmd = 'cp'
    StoreControlClient().s3cmd(cmd, source, target, recursive=recursive)

@click.command()
@click.argument('source', default='')
@click.option('--recursive', '-r', flag_value=bool, default=False, help='recursive upload')
@click.pass_context
def rm(ctx, source, recursive):
    """ Remove files from Snark Storage """
    StoreControlClient().s3cmd('rm', source, recursive=recursive)

@click.command()
@click.argument('source', default='')
@click.argument('target', default='')
@click.option('--recursive', '-r', flag_value=bool, default=False, help='recursive upload')
@click.pass_context
def mv(ctx, source, target, recursive):
    """Move files in Snark Storage """
    StoreControlClient().s3cmd('mv', source, target,  recursive=recursive)



from click import ClickException


class SnarkException(ClickException):

    def __init__(self, message=None, code=None):
        super(SnarkException, self).__init__(message)


class AuthenticationException(SnarkException):

    def __init__(self, message="Authentication failed. Please login again."):
        super(AuthenticationException, self).__init__(message=message)


class AuthorizationException(SnarkException):

    def __init__(self, response):
        try:
            message = response.json()["message"]
        except (KeyError, AttributeError):
            message = "You are not authorized to access this resource on Snark AI."
        super(AuthorizationException, self).__init__(message=message)


class NotFoundException(SnarkException):

    def __init__(self, message="The resource you are looking for was not found. Check if the name or id is correct."):
        super(NotFoundException, self).__init__(message=message)


class BadRequestException(SnarkException):

    def __init__(self, response):
        try:
            message = "One or more request parameters is incorrect\n%s" % response.json()['message']
        except (KeyError, AttributeError):
            message = "One or more request parameters is incorrect, %s" % response.content
        super(BadRequestException, self).__init__(message=message)


class OverLimitException(SnarkException):

    def __init__(self, message="You are over the allowed limits for this operation. Consider upgrading your account."):
        super(OverLimitException, self).__init__(message=message)


class ServerException(SnarkException):

    def __init__(self, message="Internal Snark AI server error."):
        super(ServerException, self).__init__(message=message)


class BadGatewayException(SnarkException):

    def __init__(self, message="Invalid response from Snark AI server."):
        super(BadGatewayException, self).__init__(message=message)


class GatewayTimeoutException(SnarkException):

    def __init__(self, message="Snark AI server took too long to respond."):
        super(GatewayTimeoutException, self).__init__(message=message)


class WaitTimeoutException(SnarkException):

    def __init__(self, message="Timeout waiting for server state update."):
        super(WaitTimeoutException, self).__init__(message=message)


class LockedException(SnarkException):

    def __init__(self, message="Resource locked."):
        super(LockedException, self).__init__(message=message)



import snark
import pkg_resources
from outdated import check_outdated, warn_if_outdated
from subprocess import call
import subprocess
import os
from snark.exceptions import SnarkException

def get_cli_version():
    return "1.0.0" #TODO

def verify_cli_version():
    try:
        version = pkg_resources.get_distribution(snark.__name__).version
        is_outdated, latest_version = check_outdated(snark.__name__, version)
        if is_outdated:
            print('\033[93m'+"Snark is out of date. Please upgrade the package by running `pip3 install --upgrade snark`"+'\033[0m')
    except:
        pass

def check_program_exists(command):
    try:
        subprocess.call([command])
    except OSError as e:
        if e.errno == os.errno.ENOENT:
            return False
        else:
            # Something else went wrong while trying to run `wget`
            return True
    return True

def get_proxy_command(proxy):
    ssh_proxy = ""
    if proxy and proxy != " " and proxy != "None" and proxy != "":
        if check_program_exists('ncat'):
            ssh_proxy = '-o "ProxyCommand=ncat --proxy-type socks5 --proxy {} %h %p"'.format(proxy)
        else:
            raise SnarkException(message="This pod is behind the firewall. You need one more thing. Please install nmap by running `sudo apt-get install nmap` on Ubuntu or `brew install nmap` for Mac")
    return ssh_proxy



import click
from snark.log import logger
from snark.client.hyper_control import HyperControlClient
from snark.client.store_control import StoreControlClient
import yaml
from os import walk
import pprint
from tabulate import tabulate
import json
import os

def get_all_files(path = "./"):
    f = []
    for (dirpath, dirnames, filenames) in walk(path):
        f.append((dirpath, dirnames, filenames))
        #break
    #print(f)
    return f

@click.command()
@click.argument('name_id', default='my_experiment')
@click.option('--file', '-f', default='snark.yml', help='YAML descriptor file for uploading')
@click.pass_context
def up(ctx, file, name_id):
    """ Start experiment """
    get_all_files()
    with open(file, 'r') as stream:
        data_loaded = yaml.load(stream)
    descriptor = open(file,'rb')
    print("Creating Experiments...")
    exp_dict = HyperControlClient().upload(descriptor)
    print("Starting Cluster...")
    show_experiments(exp_dict)

@click.command()
@click.argument('name_id', default='')
@click.option('--file', '-f', default='snark.yml', help='YAML descriptor file for uploading')
@click.pass_context
def down(ctx, file, name_id):
    """ Terminate experiment """
    if name_id == '':
       print('please provide experiment id')
       exit()

    exp = HyperControlClient().list_task(name_id)
    if not 'ID' in exp:
       print('provided experiment id is wrong')
       exit()
    pods_dict = HyperControlClient().down(exp['ID'])


@click.command()
@click.argument('name_id', default='')
@click.option('--all', '-a', flag_value=bool, default=False, help='YAML descriptor file for uploading')
@click.pass_context
def ps(ctx, name_id, all):
   """ List experiments """
   if name_id == "":
        experiments = HyperControlClient().list(all)

        experiments = [[e['ID'][:6], e['Name'], e['State'], e['IPs'], len(e['Tasks'])]
                 for e in experiments]
        print (tabulate(experiments, headers=['ID', 'Name', 'State', 'IPs', 'Tasks']))
   else:
       exp = HyperControlClient().list_experiment(name_id)
       show_tasks(exp)


def show_experiments(experiments):
    experiments = [[e['ID'][:6], e['Name'], e['State'], e['IPs'], len(e['Tasks'])]
                    for e in experiments]
    print (tabulate(experiments, headers=['ID', 'Name', 'State', 'IPs', 'Tasks']))

def show_tasks(exp):
    if 'Tasks' in exp:
        tasks = exp['Tasks']
        task_list = [[t['TaskId'][:6], t['State'], t['AssignedNode']]
                    for t in tasks]
        print (tabulate(task_list, headers=['Name', 'State', 'Node']))
    else:
         print('Task was not found')

@click.command()
@click.argument('name_id', default='')
@click.pass_context
def logs(ctx, name_id):
   """ Get logs of task by task_id"""
   if name_id == '':
       print('please provide task id')
       exit()

   exp = HyperControlClient().list_task(name_id)

   if not 'Tasks' in exp:
       print('No Task found by provided ID')
       return

   tasks = exp['Tasks']

   for task in tasks:
       if task['TaskId'][:6] == name_id:
           if len(task['Logs'])>0:
               log =str(json.loads(task['Logs'][0])['result'])
               log = log.replace('\\r\\n', '\n').replace('\\r','\n')[2:-1]
               print(log)
           else:
               print("No Logs: No task was executed")

@click.command()
@click.argument('source', default='')
@click.argument('target', default='')
@click.pass_context
def sync(ctx, source, target):
    StoreControlClient().s3cmd('sync', source, target)

@click.command()
@click.argument('source', default='')
@click.pass_context
def ls(ctx, source):
    StoreControlClient().s3cmd('ls', source)



import click
import sys

from snark.cli.auth          import login, logout
#from snark.cli.pod_control   import start, stop, ls
from snark.cli.data_transfer import push, pull
from snark.cli.connect       import attach
from snark.cli.hyper            import up, down, ps, logs
from snark.cli.store            import ls, sync, cp, rm, mv
from snark import config
from snark.log import configure_logger

@click.group()
@click.option('-h', '--host', default='{}'.format(config.SNARK_REST_ENDPOINT), help='Snark server rest endpoint')
@click.option('-v', '--verbose', count=True, help='Devel debugging')
def cli(host, verbose):
    configure_logger(verbose)

def add_commands(cli):
    cli.add_command(login)
    cli.add_command(logout)

    cli.add_command(up)
    cli.add_command(down)
    cli.add_command(ps)
    cli.add_command(logs)

    cli.add_command(ls)
    cli.add_command(sync)
    cli.add_command(cp)
    cli.add_command(rm)
    cli.add_command(mv)

add_commands(cli)



import click

import snark
from snark import config
from snark.log import configure_logger
from snark.main import add_commands


@click.group()
@click.option('-v', '--verbose', count=True, help='Devel debugging')
def cli(verbose):
    config.SNARK_REST_ENDPOINT = "http://127.0.0.1:5000"
    config.SNARK_HYPER_ENDPOINT = "http://127.0.0.1:8000"
    configure_logger(verbose)

add_commands(cli)



import click
from snark.log import logger
from snark import config
from snark.client.auth import AuthClient
from snark.token_manager import TokenManager
from snark.client.store_control import StoreControlClient

@click.command()
#@click.option('--token', is_flag=True, default=False,
#        help='Enter authentication tocken from {}'.format(config.GET_TOKEN_REST_SUFFIX))
@click.option('--username', '-u', default=None, help='Your Snark Ai username')
@click.option('--password', '-p', default=None, help='Your Snark AI password')
def login(username, password):
    """ Logs in to Snark AI"""
    token = ''
    if token:
        logger.info("Token login.")
        logger.degug("Getting the token...")
        token = click.prompt('Please paste the authentication token from {}'.format(config.GET_TOKEN_REST_SUFFIX,
            type=str, hide_input=True))
        token = token.strip()
        is_valid = AuthClient.check_token(token)
    else:
        logger.info("Please log in using Snark AI credentials. You can register at https://lab.snark.ai ")
        if not username:
            logger.debug("Prompting for username.")
            username = click.prompt('Username', type=str)
        username = username.strip()
        if not password:
            logger.debug("Prompting for password.")
            password = click.prompt('Password', type=str, hide_input=True)
        password = password.strip()
        token = AuthClient().get_access_token(username, password)
    TokenManager.set_token(token)
    StoreControlClient().get_credentials()
    logger.info("Login Successful.")

@click.command()
def logout():
    """ Logs out of Snark AI"""
    TokenManager.purge_token()



import click

import snark
from snark import config
from snark.log import configure_logger
from snark.main import add_commands


@click.group()
@click.option('-v', '--verbose', count=True, help='Devel debugging')
def cli(verbose):
    config.SNARK_REST_ENDPOINT = "https://controller.snark.ai"
    config.SNARK_HYPER_ENDPOINT = "https://dev.hyper.snark.ai"
    configure_logger(verbose)

add_commands(cli)



#!/usr/bin/env python
import os
from setuptools import find_packages, setup

project = "snark"
version = "0.4.3.1"

#TODO: add description

setup(
    name=project,
    version=version,
    description="Snark AI command line client",
    author="Snark AI Inc.",
    author_email="support@snarkai.com",
    url="https://github.com/snarkai/snark-cli",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    keywords="snark",
    python_requires='>=3',
    install_requires=[
        "click>=6.7,<7",
        "clint>=0.5.1,<1",
        "requests>=2.12.4,<3",
        "requests-toolbelt>=0.7.1,<1",
        "marshmallow>=2.11.1,<3.0.0b0",
        "pytz>=2016.10",
        "tabulate>=0.7.7,<1",
        "outdated",
        "pyyaml",
        "awscli"
    ],
    setup_requires=[],
    dependency_links=[],
    entry_points={
        "console_scripts": [
            "snark = snark.main:cli",
            "snark-local = snark.development.local:cli",
            "snark-dev = snark.development.dev:cli",
        ],
    },
    tests_require=[
        "pytest",
        "mock>=1.0.1",
    ],
)



import requests
import sys

import snark
from snark import config
from snark.log import logger
from snark.token_manager import TokenManager
from snark.cli.utils import get_cli_version

from snark.exceptions import (AuthenticationException, AuthorizationException,
                              BadGatewayException, BadRequestException,
                              GatewayTimeoutException, NotFoundException,
                              OverLimitException, ServerException,
                              LockedException, SnarkException)

def urljoin(*args):
    """
    Joins given arguments into a url. Trailing but not leading slashes are
    stripped for each argument.
    """
    return "/".join(map(lambda x: str(x).strip('/'), args))

class SnarkHttpClient(object):
    """
    Basic communication with Snark AI Controller rest API
    """
    def __init__(self):
        self.auth_header = TokenManager.get_auth_header()

    def request(self, method, relative_url, endpoint=None,
                params={}, data={}, files={}, json={}, timeout=config.DEFAULT_TIMEOUT, headers={}):
        if not endpoint:
            endpoint = config.SNARK_REST_ENDPOINT
            
        request_url = urljoin(endpoint, relative_url)
        headers['snark-cli-version'] = get_cli_version()
        if not 'Authorization' in headers or headers['Authorization'] != self.auth_header:
            headers['Authorization'] = self.auth_header

        try:
            logger.debug("Sending: Headers {}, Json: {}".format(headers, json))
            response = requests.request(method,
                                       request_url,
                                       params = params,
                                       data = data,
                                       json=json,
                                       headers=headers,
                                       files=files,
                                       timeout=timeout)
        except requests.exceptions.ConnectionError as e:
           logger.debug("Exception: {}".format(e, exc_info=True))
           sys.exit("Connection error. Please retry or check your internet connection")
        except requests.exceptions.Timeout as e:
           logger.debug("Exception: {}".format(e, exc_info=True))
           sys.exit("Connection timeout. Please retry or check your internet connection")
        logger.debug("Response Content: {}, Headers: {}".format(response.content, response.headers))
        self.check_response_status(response)
        return response

    def check_response_status(self, response):
        """
        Check response status and throw corresponding exception on failure
        """
        code = response.status_code

        if code < 200 or code >= 300:
            try:
                message = response.json()["error"]
            except Exception:
                message = None

            logger.debug('Error received: status code: {}, message: "{}"'.format(response.status_code,
                                                                                 message))
            if code == 400:
                raise BadRequestException(response)
            elif response.status_code == 401:
                raise AuthenticationException()
            elif response.status_code == 403:
                raise AuthorizationException()
            elif response.status_code == 404:
                raise NotFoundException()
            elif response.status_code == 429:
                raise OverLimitException(message)
            elif response.status_code == 502:
                raise BadGatewayException()
            elif response.status_code == 504:
                raise GatewayTimeoutException(message)
            elif response.status_code == 423:
                raise LockedException(message)
            elif 500 <= response.status_code < 600:
                if 'Server under maintenance' in response.content.decode():
                    raise ServerException('Server under maintenance, please try again later.')
                else:
                    raise ServerException()
            else:
                msg = "An error occurred. Server response: {}".format(response.status_code)
                raise SnarkException(message=msg)
